/*  Boolector: Satisfiablity Modulo Theories (SMT) solver.
 *
 *  Copyright (C) 2011-2012 Armin Biere.
 *
 *  All rights reserved.
 *
 *  This file is part of Boolector.
 *  See COPYING for more information on using this software.
 */

#ifndef BTORSMT2_H_INLCUDED
#define BTORSMT2_H_INLCUDED

#include "btorparse.h"
#include <stdio.h>

typedef BtorTerm (*btorsmt2_noarg_fun_type)(void * context);
typedef BtorTerm (*btorsmt2_unary_fun_type)(void * context, BtorTerm x);
typedef BtorTerm (*btorsmt2_binary_fun_type)(void * context, BtorTerm x, BtorTerm y);
typedef BtorTerm (*btorsmt2_ternary_fun_type)(void * context, BtorTerm x, BtorTerm y, BtorTerm z);
typedef BtorTerm (*btorsmt2_extend_fun_type)(void *, BtorTerm, int);
typedef BtorTerm (*btorsmt2_extract_fun_type)(void *, BtorTerm, int, int);
typedef BtorTerm (*btorsmt2_var_fun_type)(void *, int, char*);
typedef BtorTerm (*btorsmt2_const_fun_type)(void *, char*);
typedef BtorTerm (*btorsmt2_intarg_fun_type)(void *, int);
typedef void (*btorsmt2_void_fun_type)(void *, BtorTerm);
typedef int (*btorsmt2_int_fun_type)(void *, BtorTerm);

typedef struct BtorSMT2ParseOpt {
	void * context;

	BtorParseMode incremental;
	int verbosity;
	int need_model;
	int window;

	btorsmt2_var_fun_type var_exp;
	btorsmt2_const_fun_type const_exp;

	btorsmt2_noarg_fun_type true_exp, false_exp;
	btorsmt2_intarg_fun_type zero_exp;

	btorsmt2_unary_fun_type bvnot_exp;
	btorsmt2_binary_fun_type bvor_exp, bvnor_exp, bvand_exp, bvnand_exp, bvxor_exp, bvxnor_exp, implies_exp;
	btorsmt2_ternary_fun_type ite_exp;

	btorsmt2_binary_fun_type eq_exp, distinct_exp;
	btorsmt2_binary_fun_type bvult_exp, bvule_exp, bvugt_exp, bvuge_exp;
	btorsmt2_binary_fun_type bvslt_exp, bvsle_exp, bvsgt_exp, bvsge_exp;

	btorsmt2_unary_fun_type bvneg_exp;
	btorsmt2_binary_fun_type bvadd_exp, bvsub_exp, bvmul_exp;
	btorsmt2_binary_fun_type bvudiv_exp, bvurem_exp;
	btorsmt2_binary_fun_type bvsdiv_exp, bvsrem_exp, bvsmod_exp;

	btorsmt2_binary_fun_type bvshl_exp, bvlshr_exp, bvashr_exp;

	btorsmt2_binary_fun_type concat_exp;
	btorsmt2_extract_fun_type extract_exp;
	btorsmt2_extend_fun_type zero_extend_exp, sign_extend_exp;

	btorsmt2_int_fun_type get_exp_len;
	btorsmt2_unary_fun_type copy_exp;
	btorsmt2_void_fun_type release_exp;

	btorsmt2_int_fun_type is_array;

	// for QF_BV these functions are not needed
	BtorTerm (*array_exp)(void * context, int width, int domain, char * str);
	BtorTerm (*read_exp)(void * context, BtorTerm x, BtorTerm y);
	BtorTerm (*write_exp)(void * context, BtorTerm x, BtorTerm y, BtorTerm z);
	int (*get_index_exp_len)(void * context, BtorTerm x);
	BtorTerm (*redor_exp)(void * context, BtorTerm x);
} BtorSMT2ParseOpt;

const BtorParserAPI * btor_smt2_parser_api();

#endif
