/*  BV2SMV: Tool for Translating QF_BV problems into SMV.
 *
 *  Copyright (C) 2013 Andreas Fröhlich and Gergely Kovásznai, FMV, JKU Linz.
 *
 *  All rights reserved.
 *
 *  This file is part of BV2SMV.
 *  See COPYING for more information on using this software.
 */

#ifndef BV2SMV_HASH_H_
#define BV2SMV_HASH_H_

#include "btorsmt2/btormem.h"

/*
 * Function pointer types for hash, comparison, and deletion functions
 */
typedef unsigned int (*bv2smv_hash_fun)(const void *key);
typedef int (*bv2smv_hashCmp_fun)(const void *a, const void *b);
//typedef void (*bv2smv_hashDelete_ptr) (bv2smv_MemMan *mm, void *key);

/*
 * Type for data assigned to keys
 */
typedef union {
	int asInt;
	void * asPtr;
//  char * asStr;
} bv2smv_hashData;

/*
 * Type for slots (buckets) in the hashtable
 */
typedef struct bv2smv_hashBucket {
	/* public:
	 */
	const void *key;

	bv2smv_hashData data;

	struct bv2smv_hashBucket *next; /* chronologically */
	struct bv2smv_hashBucket *prev; /* chronologically */

	/* private:
	 */
	struct bv2smv_hashBucket *chain;/* collision chain */
} bv2smv_hashBucket;

/*
 * Type for hashtables
 */
typedef struct {
	BtorMemMgr *mem;

	unsigned size;
	unsigned count;
	bv2smv_hashBucket **table;

	bv2smv_hash_fun hash;
	bv2smv_hashCmp_fun cmp;
//  DeletePtr del;

	bv2smv_hashBucket *first;/* chronologically */
	bv2smv_hashBucket *last; /* chronologically */
} bv2smv_hashTable;

bv2smv_hashTable * bv2smv_hashTable_new(BtorMemMgr *, bv2smv_hash_fun, bv2smv_hashCmp_fun/*, DeletePtr*/);

void bv2smv_hashTable_delete(bv2smv_hashTable * p2iht);

bv2smv_hashBucket * bv2smv_hashTable_find(bv2smv_hashTable *, const void *);

bv2smv_hashBucket * bv2smv_hashTable_insert(bv2smv_hashTable *, const void *key);

//void removeFromHashTable (HashTable *, void * key, void ** stored_key_ptr, HashData * stored_data_ptr);

bv2smv_hashTable * bv2smv_strHashTable_new(BtorMemMgr * mm);
void * bv2smv_strHashTable_find(bv2smv_hashTable * t, const char * bvName);

bv2smv_hashTable * bv2smv_uintHashTable_new(BtorMemMgr * mm);
bv2smv_hashTable * bv2smv_ulongHashTable_new(BtorMemMgr * mm);

#endif /* BV2SMV_HASH_H_ */
