/*  BV2SMV: Tool for Translating QF_BV problems into SMV.
 *
 *  Copyright (C) 2013 Andreas Fröhlich and Gergely Kovásznai, FMV, JKU Linz.
 *
 *  All rights reserved.
 *
 *  This file is part of BV2SMV.
 *  See COPYING for more information on using this software.
 */

#include "helper.h"
#include "btorsmt2/btormem.h"
#include "assert.h"
#include <math.h>
#include <stdarg.h>
#include <ctype.h>

unsigned int ceillog10(unsigned long int x) {
	return ceil(log10(x));
}

char * strCopy(BtorMemMgr * mm, const char * s) {
	char * t;
	BTOR_NEWN(mm, t, strlen(s)+1);

	return strcpy(t, s);
}

char * strConcat(BtorMemMgr * mm, char delimiter, unsigned int count, ...) {
	char * res;

	va_list tokens;
	unsigned int size = 0;
	char * token;

	va_start(tokens, count);
	unsigned int i;
	for (i = 0; i < count; i++)
		if ((token = va_arg(tokens, char*) ))
			size += strlen(token) + 1;
	va_end(tokens);

	BTOR_NEWN(mm, res, size);
	size = 0;
	va_start(tokens, count);
	for (; count; count--) {
		if ((token = va_arg(tokens, char*) )) {
			strcpy(res + size, token);
			size += strlen(token) + 1;
		}
		res[size - 1] = delimiter;
	}
	res[size - 1] = 0;
	va_end(tokens);

	return res;
}

void strOverwriteSpaces(char * s, char c) {
	for (; *s; s++)
		if (isspace(*s))
			*s = c;
}

unsigned long int parseStrBin(const char * s) {
	unsigned long int x;

	for (x = 0; *s; s++) {
		x <<= 1;
		if (*s != '0')
			x |= 1;
	}

	return x;
}
