/*  BV2SMV: Tool for Translating QF_BV problems into SMV.
 *
 *  Copyright (C) 2013 Andreas Fröhlich and Gergely Kovásznai, FMV, JKU Linz.
 *
 *  All rights reserved.
 *
 *  This file is part of BV2SMV.
 *  See COPYING for more information on using this software.
 */

#ifndef BV2SMV_SMT2_H_
#define BV2SMV_SMT2_H_

#include "bv2smv.h"
#include "btorsmt2/btorsmt2.h"

unsigned int bv2smv_get_bvWidth(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x);

bv2smv_term * bv2smv_term_var(BtorSMT2ParseOpt * parseOpt, unsigned int bvWidth, const char * bvName);
bv2smv_term * bv2smv_term_const(BtorSMT2ParseOpt * parseOpt, char * str);
bv2smv_term * bv2smv_term_true(BtorSMT2ParseOpt * parseOpt);
bv2smv_term * bv2smv_term_false(BtorSMT2ParseOpt * parseOpt);
bv2smv_term * bv2smv_term_zero(BtorSMT2ParseOpt * parseOpt, unsigned int bvWidth);

bv2smv_term * bv2smv_term_bvnot(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x);
bv2smv_term * bv2smv_term_bvand(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvnand(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvor(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvnor(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvxor(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvxnor(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_implies(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_ite(BtorSMT2ParseOpt * parseOpt, bv2smv_term * cond, bv2smv_term * x, bv2smv_term * y);

bv2smv_term * bv2smv_term_bvcomp(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_distinct(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvult(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvugt(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvule(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvuge(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvslt(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvsgt(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvsle(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvsge(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);

bv2smv_term * bv2smv_term_bvadd(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvneg(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x);
bv2smv_term * bv2smv_term_bvsub(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvmul(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvudiv(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvurem(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvsdiv(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvsrem(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvsmod(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);

bv2smv_term * bv2smv_term_extract(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, unsigned int indUpper, unsigned int indLower);
bv2smv_term * bv2smv_term_concat(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_zero_extend(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, unsigned int cntNewBits);
bv2smv_term * bv2smv_term_sign_extend(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, unsigned int cntNewBits);

bv2smv_term * bv2smv_term_bvshl(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvlshr(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);
bv2smv_term * bv2smv_term_bvashr(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x, bv2smv_term * y);

// dummy functions, needed by btorsmt2
bv2smv_term * bv2smv_copy_term(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x);
void bv2smv_release_term(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x);
int bv2smv_is_array_term(BtorSMT2ParseOpt * parseOpt, bv2smv_term * x);

#endif /* BV2SMV_SMT2_H_ */
